package ink.wgink.module.activiti.controller.app.api.oa;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.activiti.service.oa.IOaFormReportService;
import ink.wgink.module.activiti.util.oa.OaFormReportUtil;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: OAFormReportController
 * @Description: OA表单上报
 * @Author: wanggeng
 * @Date: 2022/3/22 21:55
 * @Version: 1.0
 */
@Api(tags = ISystemConstant.API_TAGS_APP_ROUTE_PREFIX + "oa表单上报")
@RestController
@RequestMapping(ISystemConstant.APP_PREFIX + "/oa-form-report")
public class OaFormReportAppController extends DefaultBaseController {

    @Autowired
    private IOaFormReportService oaFormReportService;

    @ApiOperation(value = "保存表单（发起流程）", notes = "保存表单（发起流程）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "processDefinitionId", value = "流程定义ID", paramType = "path"),
            @ApiImplicitParam(name = "formCode", value = "表单编码", paramType = "path"),
            @ApiImplicitParam(name = "formVersion", value = "表单版本", paramType = "path")
    })
    @PostMapping("save/definition-id/{processDefinitionId}/code/{formCode}/version/{formVersion}")
    public SuccessResult save(@RequestHeader("token") String token,
                              @PathVariable("processDefinitionId") String processDefinitionId,
                              @PathVariable("formCode") String formCode,
                              @PathVariable("formVersion") Integer formVersion,
                              @RequestBody Map<String, Object> params) {
        OaFormReportUtil.checkReportBody(params);
        oaFormReportService.saveByToken(token, processDefinitionId, formCode, formVersion, params);
        return new SuccessResult();
    }

    @ApiOperation(value = "更新表单", notes = "更新表单接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "taskId", value = "任务ID", paramType = "path"),
            @ApiImplicitParam(name = "formCode", value = "表单编码", paramType = "path"),
            @ApiImplicitParam(name = "formVersion", value = "表单版本", paramType = "path"),
            @ApiImplicitParam(name = "uid", value = "表单主键", paramType = "path")
    })
    @PutMapping("update/task-id/{taskId}/code/{formCode}/version/{formVersion}/is-need-claim/{isNeedClaim}/uid/{uid}")
    public SuccessResult update(@RequestHeader("token") String token,
                                @PathVariable("taskId") String taskId,
                                @PathVariable("formCode") String formCode,
                                @PathVariable("formVersion") Integer formVersion,
                                @PathVariable("isNeedClaim") Integer isNeedClaim,
                                @PathVariable("uid") String uid,
                                @RequestBody Map<String, Object> params) {
        if (isNeedClaim != 0 && isNeedClaim != 1) {
            throw new ParamsException("认领参数错误");
        }
        OaFormReportUtil.checkReportBody(params);
        oaFormReportService.updateByToken(token, taskId, formCode, formVersion, isNeedClaim, uid, params);
        return new SuccessResult();
    }

    @ApiOperation(value = "查询表单", notes = "更新表单接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "formCode", value = "表单编码", paramType = "path"),
            @ApiImplicitParam(name = "formVersion", value = "表单版本", paramType = "path"),
            @ApiImplicitParam(name = "uid", value = "表单主键", paramType = "path")
    })
    @GetMapping("get/code/{formCode}/version/{formVersion}/uid/{uid}")
    public Map<String, Object> get(@RequestHeader("token") String token,
                                   @PathVariable("formCode") String formCode,
                                   @PathVariable("formVersion") Integer formVersion,
                                   @PathVariable("uid") String uid) {
        return oaFormReportService.get(formCode, formVersion, uid);
    }

    @ApiOperation(value = "表单列表", notes = "表单列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "formCode", value = "表单编码", paramType = "path"),
            @ApiImplicitParam(name = "formVersion", value = "表单版本", paramType = "path"),
            @ApiImplicitParam(name = "uid", value = "表单主键", paramType = "path")
    })
    @GetMapping("list/code/{formCode}/version/{formVersion}")
    public List<Map<String, Object>> list(@RequestHeader("token") String token,
                                          @PathVariable("formCode") String formCode,
                                          @PathVariable("formVersion") Integer formVersion) {
        Map<String, Object> params = requestParams();
        return oaFormReportService.list(formCode, formVersion, params);
    }

    @ApiOperation(value = "表单分页列表", notes = "表单分页列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "formCode", value = "表单编码", paramType = "path"),
            @ApiImplicitParam(name = "formVersion", value = "表单版本", paramType = "path"),
            @ApiImplicitParam(name = "uid", value = "表单主键", paramType = "path")
    })
    @GetMapping("listpage/code/{formCode}/version/{formVersion}")
    public SuccessResultList<List<Map<String, Object>>> listPage(@RequestHeader("token") String token,
                                                                 @PathVariable("formCode") String formCode,
                                                                 @PathVariable("formVersion") Integer formVersion,
                                                                 ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return oaFormReportService.listPage(formCode, formVersion, page);
    }
}
